'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var mitt = require('mitt');
var WS = require('ws');
var UUID = require('uuid');
var reactivity = require('@vue/reactivity');
var runtimeCore = require('@vue/runtime-core');
var deepmerge = require('deepmerge');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        for (var k in e) {
            n[k] = e[k];
        }
    }
    n["default"] = e;
    return Object.freeze(n);
}

var mitt__default = /*#__PURE__*/_interopDefaultLegacy(mitt);
var WS__default = /*#__PURE__*/_interopDefaultLegacy(WS);
var UUID__namespace = /*#__PURE__*/_interopNamespace(UUID);
var deepmerge__default = /*#__PURE__*/_interopDefaultLegacy(deepmerge);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const emitter$1 = mitt__default();
class SockSend {
    constructor(options) {
        this.sendCmdCache = {}; // 缓存的接口
        this.on = emitter$1.on;
        this.options = options;
    }
    /**
     * 注册事件 把事件存储起来 client -> serve  向serve通信
     * @params cmd 指令
     * @params data 配置项
     * @params data.uuid? 任务id
     * @params data.content? 数据
     * @params data.options? 配置项
     * @params data.options.isOff 是否要被主动注销
     * @params data.options.isCacheCmd 是否缓存
     * @params data.options.timeout 超时时间
     * @params callback({req, res, socket}) 回调函数
     * */
    send(cmd, data, callback) {
        const sendOptions = Object.assign({ isOff: false }, data === null || data === void 0 ? void 0 : data.options);
        const uuid = (data === null || data === void 0 ? void 0 : data.uuid) || UUID__namespace.v1();
        // cmd里获取需传输的数据格式
        const req = { uuid, cmd, content: data === null || data === void 0 ? void 0 : data.content };
        return new Promise(((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const sendCacheItem = {
                req,
                callback,
                resolve,
                reject,
                options: sendOptions,
                status: 'pending', // 'pending' 请求中 | 'resolved' 已完成 | 'rejected' 已失败
            };
            // 如果配置项包含超时
            if (sendOptions.timeout) {
                sendCacheItem.onTimeout = setTimeout(() => {
                    this.sendError(`连接超时：${this.options.url}; cmd: ${cmd}; ${sendOptions.timeout}ms`, uuid);
                }, sendOptions.timeout);
            }
            // 发送消息
            this.options.clientSend(req);
            ((_a = data === null || data === void 0 ? void 0 : data.options) === null || _a === void 0 ? void 0 : _a.onSendBefore) && data.options.onSendBefore(req);
            this.sendCmdCache[uuid] = sendCacheItem;
        })));
    }
    // 可能是所有接口 或 某个接口 发送错误消息
    sendError(error_msg, uuid) {
        var _a;
        if (!uuid) {
            for (let uuid in this.sendCmdCache) {
                this.sendError(error_msg, uuid);
            }
        }
        else {
            const sendCmdCacheItem = this.sendCmdCache[uuid];
            // 将正在运行中状态变为失败
            if ((sendCmdCacheItem === null || sendCmdCacheItem === void 0 ? void 0 : sendCmdCacheItem.status) === 'pending') {
                this.sendCmdCache[uuid].status = 'rejected';
            }
            const resData = {
                uuid,
                cmd: (_a = sendCmdCacheItem === null || sendCmdCacheItem === void 0 ? void 0 : sendCmdCacheItem.req) === null || _a === void 0 ? void 0 : _a.cmd,
                error_msg,
                content: {},
                status: -1
            };
            this.emit(resData);
        }
    }
    emit(res) {
        const { cmd, uuid, content = {} } = res;
        const sendCmdCacheItem = this.sendCmdCache[uuid];
        if ((sendCmdCacheItem === null || sendCmdCacheItem === void 0 ? void 0 : sendCmdCacheItem.status) === 'pending') {
            sendCmdCacheItem.status = 'resolved'; // 已成功返回
        }
        // 如果有超时 则清除超时定时器
        (sendCmdCacheItem === null || sendCmdCacheItem === void 0 ? void 0 : sendCmdCacheItem.onTimeout) && clearTimeout(sendCmdCacheItem === null || sendCmdCacheItem === void 0 ? void 0 : sendCmdCacheItem.onTimeout);
        const SendCallbackData = {
            req: sendCmdCacheItem === null || sendCmdCacheItem === void 0 ? void 0 : sendCmdCacheItem.req,
            res,
        };
        // 订阅方式 发布cmd接口
        // 服务端主动推送
        // serve -> client
        emitter$1.emit(cmd, SendCallbackData);
        // 客户端触发  服务端返回
        // client -> serve -> client
        if (!!sendCmdCacheItem) {
            // 回调方式
            sendCmdCacheItem.callback && sendCmdCacheItem.callback(SendCallbackData);
            // 同步方式 只会接收一次数据
            sendCmdCacheItem.resolve && sendCmdCacheItem.resolve(SendCallbackData);
            // 调用一次就注销
            sendCmdCacheItem.options.isOff && this.off(uuid);
        }
    }
    // 通过uuid注销事件 避免数据臃肿
    off(uuid) {
        if (uuid) {
            delete this.sendCmdCache[uuid];
        }
        else {
            this.sendCmdCache = {};
            emitter$1.off("*");
        }
    }
}

const emitter = mitt__default();
const isNodeEnv = typeof window === 'undefined' || !globalThis.window;
class ClientSocket {
    constructor(options) {
        this.options = {
            url: '',
            logger: console,
            // socket的相关配置
            socketOptions: {
                wsConfig: isNodeEnv ? {} : undefined,
                reconnectMax: 3,
                reconnectTime: 2 * 1000,
                heartCmd: 'heart',
                heartTime: 5 * 1000, // 心跳时间
            },
            // socket 通信相关配置
            socketSendOptions: {
                // 请求拦截器
                reqInterceptors: (data) => {
                    return Buffer.from(JSON.stringify(data));
                },
                // 响应拦截器
                resInterceptors: (data) => {
                    return JSON.parse(data.toString());
                },
            },
        };
        // 主要用来获取ws的生命周期
        this.on = emitter.on;
        this.socketReadyState = {
            0: 'websocket连接尚未建立, 无法连接到服务端',
            1: 'websocket连接已建立，可以进行通信',
            2: 'websocket连接正在进行关闭',
            3: 'websocket连接已经关闭或者连接不能打开'
        };
        this.reactiveData = reactivity.reactive({
            heartTimer: undefined,
            reconnectNum: 0,
            reconnectTimer: undefined,
            isSocketReconnect: false,
            isSocketClose: false, // socket已经关闭 不会自动重启
        });
        this.socketEventMap = {
            // node event: undefined
            // web event: Event
            onopen: (event) => {
                // 启动成功后重启次数归零
                this.reactiveData.reconnectNum = 0;
                // 创建后开始心跳检测
                this.startHeart();
                emitter.emit('socket-open', event);
                // 启动成功
                emitter.emit('socket-change-state');
            },
            // node error: Error
            // web error: Event
            onerror: (error) => {
                this.options.logger.error('ws onError', error);
                emitter.emit('socket-error', error);
                emitter.emit('socket-change-state');
            },
            // node code: number, reason: Buffer
            // web code: CloseEvent
            onclose: (code, reason) => {
                var _a;
                this.stopHeart();
                this.websocketReconnect(); // 重连
                if (isNodeEnv) {
                    this.options.logger.error('ws onClose', reason && reason.toString());
                    emitter.emit('socket-close', {
                        code: code,
                        reason
                    });
                }
                else {
                    this.options.logger.error('ws onClose', (_a = code) === null || _a === void 0 ? void 0 : _a.reason);
                    emitter.emit('socket-close', code);
                }
                emitter.emit('socket-change-state');
            },
            // node data: WebSocket.RawData | MessageEvent, isBinary: boolean
            // web data: MessageEvent
            onmessage: (data) => __awaiter(this, void 0, void 0, function* () {
                //拿到任何消息都说明当前连接是正常的  重新开始心跳检测
                this.startHeart();
                const messageData = yield this.options.socketSendOptions.resInterceptors(data);
                this.sockSend.emit(messageData);
                emitter.emit('socket-message', data);
            })
        };
        // 发送消息
        this.clientSend = (req) => __awaiter(this, void 0, void 0, function* () {
            // 如果有socket
            if (this.socket && this.socket.readyState === 1) {
                const reqData = (yield this.options.socketSendOptions.reqInterceptors(req));
                this.socket.send(reqData);
            }
            else {
                this.sockSend.sendError(`websocket未创建成功 ${this.options.url}`, req.uuid);
            }
        });
        this.options.socketOptions.wsConfig = {
            handshakeTimeout: this.options.socketOptions.reconnectTime
        };
        // 深度合并
        this.options = deepmerge__default(this.options, options);
        this.sockSend = new SockSend({
            url: this.options.url,
            clientSend: this.clientSend
        });
    }
    /**
        创建websocket
     */
    createSocket() {
        return __awaiter(this, void 0, void 0, function* () {
            this.reactiveData.reconnectNum = 1;
            return new Promise((resolve) => {
                emitter.on('socket-change-state', () => {
                    var _a;
                    resolve(((_a = this.socket) === null || _a === void 0 ? void 0 : _a.readyState) || 0);
                });
                this.websocketInstance();
            });
        });
    }
    /**
     * 等待ws的启动 默认等待3次  并不会启动ws 只会等待ws启动成功
     * @params {number} reconnectMax 该次请求最多等待重启次数
     * 传递该值的情况为
     * 1.已经实例化并运行过createSocket
     * 2.readyState 不为1（ws不在运行中）
     * 3.需等待重启最多reconnectMax次
     * **/
    waitCreate(reconnectMax = 3) {
        return new Promise((resolve) => {
            var _a, _b;
            // 当前重启次数
            let restartNum = 0;
            const unWatch = runtimeCore.watch(() => this.reactiveData.reconnectNum, (newsValue, oldValue) => {
                var _a, _b, _c;
                // this.options.logger.log('newsValue, oldValue', newsValue, oldValue)
                if (newsValue - 1 === oldValue) {
                    restartNum++;
                    this.options.logger.log(`等待重启 ${this.options.url} ${restartNum}/${reconnectMax}`);
                    // 等待 reconnectMax 次后还未启动成功
                    if (restartNum >= reconnectMax) {
                        resolve(((_a = this.socket) === null || _a === void 0 ? void 0 : _a.readyState) || 0);
                        unWatch();
                    }
                    else if (this.reactiveData.isSocketClose) {
                        //  ws已经完全关闭无法重启
                        resolve(((_b = this.socket) === null || _b === void 0 ? void 0 : _b.readyState) || 0);
                        unWatch();
                    }
                }
                else if (newsValue === 0 || this.reactiveData.isSocketClose) {
                    // 启动成功 或 永久关闭后 restartNum 会变为0
                    resolve(((_c = this.socket) === null || _c === void 0 ? void 0 : _c.readyState) || 0);
                    unWatch();
                }
            }, {});
            // 如果已经启动成功
            if (this.socket) {
                if (this.socket.readyState === 1) {
                    unWatch();
                    // ws 已经启动
                    return resolve((_a = this.socket) === null || _a === void 0 ? void 0 : _a.readyState);
                }
                else if (this.reactiveData.isSocketClose) {
                    unWatch();
                    //  ws已经完全关闭无法重启
                    return resolve((_b = this.socket) === null || _b === void 0 ? void 0 : _b.readyState);
                }
            }
            else {
                this.createSocket();
            }
        });
    }
    // 实例化 websocket
    websocketInstance() {
        try {
            if (isNodeEnv) {
                this.socket = new WS__default(this.options.url, this.options.socketOptions.wsConfig);
            }
            else {
                // 浏览器环境
                this.socket = new WebSocket(this.options.url, this.options.socketOptions.wsConfig);
            }
            this.websocketEvent();
        }
        catch (e) {
            this.options.logger.log('catch', e);
            // 出错重连
            this.websocketReconnect();
        }
    }
    // 对ws重连
    websocketReconnect() {
        this.reactiveData.reconnectNum++;
        const socket = this.socket;
        if (this.reactiveData.isSocketReconnect) {
            return this.options.logger.log(`正在执行重连, ${this.options.url}`);
        }
        else if (this.reactiveData.isSocketClose) {
            return this.options.logger.log(`socket已经完全关闭后不再重启, ${this.options.url}`);
        }
        // 最多重启restartMax次(如果restartMax = 0 不重启)
        else if (((this.reactiveData.reconnectNum > this.options.socketOptions.reconnectMax) || (this.options.socketOptions.reconnectMax === 0)) && this.options.socketOptions.reconnectMax >= 0) {
            this.reactiveData.isSocketClose = true;
            const error_msg = `启动socket失败, ${this.socketReadyState[socket.readyState]}, ${socket.url}, ${this.reactiveData.reconnectNum - 1}/${this.options.socketOptions.reconnectMax}`;
            this.sockSend.sendError(error_msg);
            this.options.logger.log(error_msg);
            // 已经没法重启了
            return emitter.emit('socket-change-state');
        }
        this.reactiveData.isSocketReconnect = true;
        this.options.logger.log('重启socket', this.options.url, `${this.reactiveData.reconnectNum}/${this.options.socketOptions.reconnectMax === -1 ? '无限制' : this.options.socketOptions.reconnectMax}`);
        //没连接上会一直重连，设置延迟避免请求过多
        this.reactiveData.reconnectTimer && clearTimeout(this.reactiveData.reconnectTimer);
        this.reactiveData.reconnectTimer = setTimeout(() => {
            this.reactiveData.isSocketReconnect = false;
            this.websocketInstance();
        }, this.options.socketOptions.reconnectTime);
    }
    // 对socket的事件进行注册
    websocketEvent() {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        if (this.socket) {
            const wm = new WeakMap();
            wm.set(this.socket, this.socketEventMap);
            // this.socket 被销毁时 自动垃圾回收
            if (isNodeEnv) {
                const socket = this.socket;
                socket.addListener('open', (_a = wm.get(socket)) === null || _a === void 0 ? void 0 : _a.onopen);
                socket.addListener('error', (_b = wm.get(socket)) === null || _b === void 0 ? void 0 : _b.onerror);
                socket.addListener('close', (_c = wm.get(socket)) === null || _c === void 0 ? void 0 : _c.onclose);
                socket.addListener('message', (_d = wm.get(socket)) === null || _d === void 0 ? void 0 : _d.onmessage);
            }
            else {
                const socket = this.socket;
                socket.addEventListener('open', (_e = wm.get(socket)) === null || _e === void 0 ? void 0 : _e.onopen);
                socket.addEventListener('error', (_f = wm.get(socket)) === null || _f === void 0 ? void 0 : _f.onerror);
                socket.addEventListener('close', (_g = wm.get(socket)) === null || _g === void 0 ? void 0 : _g.onclose);
                socket.addEventListener('message', (_h = wm.get(socket)) === null || _h === void 0 ? void 0 : _h.onmessage);
            }
        }
    }
    // 开始心跳
    startHeart() {
        this.stopHeart();
        this.reactiveData.heartTimer = setInterval(() => {
            this.sockSend && this.sockSend.send(this.options.socketOptions.heartCmd);
        }, this.options.socketOptions.heartTime);
    }
    // 停止心跳
    stopHeart() {
        this.reactiveData.heartTimer && clearInterval(this.reactiveData.heartTimer);
    }
    close() {
        var _a;
        // 不再重启
        this.reactiveData.isSocketClose = true;
        emitter.off('*');
        ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.close) && this.socket.close(1000);
        this.sockSend.off();
    }
}

exports.ClientSocket = ClientSocket;
exports.SockSend = SockSend;
